# frozen_string_literal: true

#
# Cookbook:: snu_nginx
# Library:: resource/snu_nginx_conf_d
#
# Copyright:: 2019, Tyler Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require_relative 'snu_nginx_config_base'

class Chef
  class Resource
    # A custom resource for generating config files in Nginx's .d directory.
    class SnuNginxConfD < SnuNginxConfigBase
      DEFAULTS ||= {
        general: {
          default_type: 'application/octet-stream',
          server_tokens: 'off',
          sendfile: 'on',
          tcp_nopush: 'on',
          tcp_nodelay: 'on',
          keepalive_timeout: 65,
          types_hash_max_size: 1024,
          server_names_hash_bucket_size: 64,
          proxy_max_temp_file_size: 0
        },
        gzip: {
          gunzip: 'on',
          gzip: 'on',
          http_version: '1.0',
          comp_level: 2,
          proxied: 'any',
          # text/html is implicit
          types: %w[
            application/javascript
            application/json
            application/pdf
            application/rdf+xml
            application/rss+xml
            application/vnd.geo+json
            application/vnd.ms-excel
            application/x-javascript
            application/xml
            application/xml+rss
            image/bmp
            image/svg+xml
            text/css
            text/csv
            text/javascript
            text/plain
            text/xml
          ]
        },
        logging: {
          log_format: '$remote_addr - $remote_user [$time_local] "$request" ' \
                      '$status $body_bytes_sent "$http_referer" "$http_user_agent" ' \
                      '$request_length $request_time "$http_host" "" "" ' \
                      '"$ssl_protocol" "$ssl_cipher" "" "$realip_remote_addr" ' \
                      '"$http_x_forwarded_for"',
          log_dir: '/var/log/%<service_name>s'
        },
        ssl: { session_cache: 'shared:SSL:10m', session_timeout: '10m' }
      }.freeze

      provides :snu_nginx_conf_d

      # The name of this config will be used to construct its path.
      property :config_name, String, name_property: true

      # The name, cookbook, and variables to use as config template.
      property :template, String, default: lazy { |r| "#{r.config_name}.conf.erb" }
      property :cookbook, String, default: 'snu_nginx'
      property :variables,
               Hash,
               default: lazy { |r|
                 DEFAULTS[r.config_name.to_sym].to_h.each_with_object({}) do |(k, v), hsh|
                   hsh[k] = v.is_a?(String) ? format(v, service_name: r.service_name) : v
                 end
               }

      # Create the conf.d file.
      action :create do
        vars = new_resource.variables

        template ::File.join(new_resource.dir,
                             new_resource.service_name,
                             "conf.d/#{new_resource.config_name}.conf") do
          owner 'root'
          group 'root'
          mode '0644'
          source new_resource.template
          cookbook new_resource.cookbook
          variables vars
          notifies :reload, "snu_nginx_service[#{new_resource.service_name}]"
        end
      end

      # Delete the conf.d file.
      action :remove do
        file ::File.join(new_resource.dir,
                         new_resource.service_name,
                         "conf.d/#{new_resource.config_name}.conf") do
          action :delete
          notifies :reload, "snu_nginx_service[#{new_resource.service_name}]"
        end
      end
    end
  end
end
