# frozen_string_literal: true

#
# Cookbook:: snu_nginx
# Library:: resource/snu_nginx_config
#
# Copyright:: 2019, Tyler Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require_relative '../helpers/config'
require_relative 'snu_nginx_config_base'

class Chef
  class Resource
    # A custom resource for generating the main nginx.conf file.
    class SnuNginxConfig < SnuNginxConfigBase
      provides :snu_nginx_config

      # The name of this config's service will be used to construct its path.
      property :name, String, default: 'nginx'
      property :service_name, String, name_property: true

      # The name, cookbook, and variables to use as an nginx.conf template.
      property :template, String, default: 'nginx.conf.erb'
      property :cookbook, String, default: 'snu_nginx'
      property :variables,
               Hash,
               default: lazy { |r|
                 {
                   user: 'nginx',
                   worker_processes: [r.node['cpu']['total'] - 1, 1].max,
                   log_dir: ::File.join('/var/log', r.service_name),
                   pid_file: "/run/#{r.service_name}.pid",
                   worker_connections: 4096,
                   worker_rlimit_nofile: 4096,
                   config_dir: ::File.join(r.dir, r.service_name)
                 }
               }

      # To create the config:
      #   - Ensure the Nginx user exists
      #   - Ensure the config dir exists
      #   - Ensure the various .d-style dirs exist
      #   - Fix ownership of Nginx's log directory and files
      #   - Write a proper logrotate config
      #   - Write an nginx.conf
      action :create do
        vars = new_resource.variables

        user(vars[:user]) { system true }

        create_config_dirs!
        fix_log_permissions!(vars[:log_dir], vars[:user])

        template ::File.join('/etc/logrotate.d', new_resource.service_name) do
          owner 'root'
          group 'root'
          mode '0644'
          source 'logrotate.erb'
          cookbook 'snu_nginx'
          variables vars
        end

        template ::File.join(new_resource.dir, new_resource.service_name, 'nginx.conf') do
          owner 'root'
          group 'root'
          mode '0644'
          source new_resource.template
          cookbook new_resource.cookbook
          variables vars
          notifies :reload, "snu_nginx_service[#{new_resource.service_name}]"
        end
      end

      # To remove the config:
      #   - Delete the config file
      #   - Delete the logrotate config
      #   - Leave any log files in place
      #   - If any of the config dirs are empty, delete them too
      #   - Leave the Nginx user in place
      action :remove do
        [
          ::File.join(new_resource.dir, new_resource.service_name, 'nginx.conf'),
          ::File.join('/etc/logrotate.d', new_resource.service_name)
        ].each do |f|
          file(f) { action :delete }
        end

        config_dirs.each do |d|
          next unless ::File.exist?(d) && Dir.empty?(d)

          directory(d) { action :delete }
        end
      end

      action_class do
        include SnuNginxCookbook::Helpers::Config
      end
    end
  end
end
