# frozen_string_literal: true

#
# Cookbook:: snu_nginx
# Library:: resource/snu_nginx_service
#
# Copyright:: 2019, Tyler Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'chef/resource'
require 'chef/resource/service'

class Chef
  class Resource
    # A custom resource for managing Nginx services.
    class SnuNginxService < Resource
      provides(:snu_nginx_service) { false }

      default_action %i[create enable start]

      property :name, String, default: 'nginx'
      property :service_name, String, name_property: true
      property :config_dir, String, default: '/etc'
      property :config_file, String, default: lazy { |r| ::File.join(r.config_dir, r.service_name, 'nginx.conf') }
      property :file_limit, Integer, default: 10_000

      # The :create and :remove actions are platform-dependent and need to be
      # defined in the subclass resources.
      #
      # action :create do; end
      # action :remove do; end

      # Every other action gets passed on to an underlying service resource.
      Resource::Service.allowed_actions.each do |act|
        action act do
          service new_resource.service_name do
            supports restart: true, reload: true, status: true
            action act
          end

          # Bail out immediately we're trying to feed Nginx an invalid config.
          if %i[start restart reload].include?(act)
            execute 'Validate Nginx config' do
              command "ulimit -n #{new_resource.file_limit} && " \
                      "/usr/sbin/nginx -t -c #{new_resource.config_file}"
              action :nothing
              subscribes :run, "service[#{new_resource.service_name}]", :before
            end
          end
        end
      end
    end
  end
end
